import jd2xx.JD2XX;
import jd2xx.JD2XXInputStream;
import jd2xx.JD2XXOutputStream;
import java.io.IOException;
import java.lang.Thread;
import java.lang.Math;

public class SensorInterface {
	
	public static JD2XX ftdi;
	
	public SensorInterface() throws Exception {
		ftdi = new JD2XX();	
		listDevicesAndConnect();
		
		// If we make it to this point, we're connected, synchronized, and ready to start
		// communicating over SPI.
		System.out.println("Device is ready!");
	}
	
	/**
	 * Lists all connected USB FT2232 devices; exits the program if none are found.
	 * Connects to the first available device and sets it up to use the MPSSE.
	 */
	public void listDevicesAndConnect() throws IOException {
		Object[] devs = null;
		devs = ftdi.listDevicesByDescription();
		
		if(devs.length < 1) {
			System.err.println("No DLP2232M device found!");
			System.exit(1);
		}
		
		System.out.println("Found DLP2232M devices:");
		for (int i=0; i<devs.length; ++i)
			System.out.println("\t(" + i + ") " + devs[i]);
		
		ftdi.open(0);
		System.out.println("Connected to device 0.");
		
		ftdi.resetDevice();
		System.out.println("Device reset.");
		
		ftdi.setLatencyTimer(5);
		ftdi.setBitMode(0x00, 0x00);
		ftdi.setBitMode(0x00, 0x02);
		setClockSpeed100k();
		syncMPSSE();
	}
	
	/**
	 * Clears the transmit and receive buffers, and sends an invalid command to make sure
	 * that we get an error like we expect.  If something is wrong, the program exits after
	 * displaying an error message.
	 */
	public void syncMPSSE() throws IOException {
		byte syncResponse[] = new byte[2];
		
		ftdi.purge(JD2XX.PURGE_RX | JD2XX.PURGE_TX);
		
		// Send a bad command to ensure that MPSSE is synchronized
		ftdi.write(0xAA);
		ftdi.read(syncResponse);
		if( (0xFF & (int)syncResponse[0]) != 0xFA || (0xFF & (int)syncResponse[1]) != 0xAA ) {
			System.out.println("MPSSE is not synchronized");
			System.exit(1);
		} else {
			System.out.println("MPSSE is synchronized");
		}
		
	}
	
	/**
	 * Sets the speed of the SPI clock to 100 KHz. (See page 19 of FTDI Application Note 108)
	 */
	public void setClockSpeed100k() throws IOException {
		byte[] dataOut = new byte[3];
		dataOut[0] = (byte)0x86;  // Set clock divisor
		dataOut[1] = (byte)0x78;  // (Clock divisor-1) low byte
		dataOut[2] = (byte)0x00;  // (Clock divisor-1) high byte
		ftdi.write(dataOut);
	}
	
	/**
	 * This method should communicate with your microcontroller to read the values
	 * of the two accumulators, then calculate and return the magnitude.  Note that
	 * this can't throw any exceptions, so make sure to catch anything that might
	 * happen.
	 */
	public int readSensor() {
		// implement me
		return 0;
	}
	
	/**
	 * This method should communicate with your microcontroller and set the red, green, and
	 * blue values of the LED to the values given in its arguments.  Note that this can't
	 * throw any exceptions, so make sure to catch anything that might happen.
	 */
	public void setLEDColor(int r, int g, int b) {
		// implement me
	}
	
	public static void main(String[] args) throws Exception {
		SensorInterface sensor = new SensorInterface();
		
		for(;;) {
			int response = sensor.readSensor();
			System.out.println("value= " + response);
		}
	}
	
}